package TauP;
use strict;
use FindBin qw($RealDir);
use lib $RealDir;
use FORKER;

our (@ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS, $VERSION);

use Exporter;
$VERSION = 2007.323;
@ISA = qw(Exporter);

@EXPORT = qw(taup_time); # symbols to autoexport (:DEFAULT tag)
@EXPORT_OK = qw(); # symbols to export on request
%EXPORT_TAGS = (
	taup => [qw(taup_time)],
	direct => [],
);

# BEGIN CODE
###################################################################


# GETS PHASE TIMES USING THE TAUP TOOLKIT
# NOTE: ONLY RETRIEVES FIRST ARRIVAL FOR A PHASE - TRIPLICATIONS ARE
#       NOT DELT WITH
sub taup_time {
	# NECESSARY PARAMETERS
	my ($rmod,$evdp,$gcarc,$phase)=@_;
	
	# RUN TAUP
	my @taup=`taup_time -mod $rmod -h $evdp -deg $gcarc -ph $phase`;
	
	# CHECK FOR FAILURE
	my $exitcode = $? >> 8;  # exit code
	my $exitsign = $? & 127; # signal that caused exit
	if ($exitcode or $exitsign) {print "RUNNING TAUP FAILED!\n" and mexit(2000);}
	
	# PARSE OUTPUT
	my @taupt=split(' ',$taup[5]); chomp(@taupt);
	
	# IF NULL OUTPUT, USE TAUP_CURVE TO RETRIEVE PHASE LIMITS AND RERUN
	if (!@taupt) {
		# RUN TAUP
		`taup_curve -mod $rmod -h $evdp -ph $phase`;
		
		# CHECK FOR FAILURE
		$exitcode = $? >> 8;  # exit code
		$exitsign = $? & 127; # signal that caused exit
		if ($exitcode or $exitsign) {print "RUNNING TAUP FAILED!\n" and mexit(2001);}
		
		# OPEN OUTPUT FILE
		open(TFILE,'taup_curve.gmt') or print "COULD NOT OPEN PHASE TRAVELTIME CURVE FILE: taup_curve.gmt!\n" and mexit(2002);
		my @curve=<TFILE>; close(TFILE); splice(@curve,0,1); @curve=map {($_)=split(' ',$_)} @curve; @curve=sort {$a<=>$b} (@curve);
		unlink('taup_curve.gmt') or print "COULD NOT DELETE FILE taup_curve.gmt!\n" and mexit(2003);
		
		# EXTRACT CLOSEST AND FARTHEST DISTANCES
		my ($near,$far)=($curve[0],$curve[$#curve]);
		
		# DECIDING WHICH TO USE
		my $newdd; if ($gcarc<$near) {$newdd=$near+0.00001;} else {$newdd=$far-0.00001;}
		
		# RUN TAUP_TIME
		my ($tt,$rp)=taup_time($rmod,$evdp,$newdd,$phase);
		
		# EXTRAPOLATING TRAVEL TIME AND RAY PARAMETER
		return $tt+($gcarc-$newdd)*$rp,$rp;
	}
	
	# RETURN OUTPUT (TRAVEL TIME (S), RAY PARAMETER (S/DEG))
	return @taupt[3..4];
}

###################################################################
1;

